/*
    DoubleSimple: DoubleEpsilon.m
    Copyright (C) 2003  Macaulay Institute

    This file is part of DoubleSimple, a series of classes for implementing
    various remedies to floating point arithmetic errors.

    DoubleSimple is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    DoubleSimple is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details. (LICENCE file in
    this directory.)

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

    Contact information:
      Gary Polhill,
      Macaulay Institute, Craigiebuckler, Aberdeen, AB15 8QH. United Kingdom
      g.polhill@macaulay.ac.uk
*/
/*

Implementation of DoubleEpsilon class

*/

#import "DoubleEpsilon.h"
#import <float.h>
#import <stdlib.h>
#import <string.h>

#ifdef INTEL
#  import <CIieeefp.h>
#endif


/* Class variable */

static double epsilon = DBL_EPSILON;

/* Implementation */

@implementation DoubleEpsilon

/* New methods for this class */

+setEpsilon: (double)e {
  if(e < 0.0) {
    fprintf(stderr, "Invalid setting for epsilon: %g\n", e);
    abort();
  }
  epsilon = e;

  return self;
}

-setEpsilon: (double)e {
  [[self class] setEpsilon: e];

  return self;
}

+(double)getEpsilon {
  return epsilon;
}

-(double)getEpsilon {
  return [[self class] getEpsilon];
}

/* Methods to override from DoubleSimple */

+parseValue: (char *)str {
  char *p = strchr(str, (int)'~');

  if(p != NULL) {
#ifdef INTEL
    fp_pctl pc = fpsetprecision(FP_PC_DBL);
#endif
    (*p) = '\0';
    p++;
    [self setEpsilon: atof(p)];
#ifdef INTEL
    fpsetprecision(pc);
#endif
  }
  return [super parseValue: str];
}

-(BOOL)eq: (DoubleSimple *)value {
  double v = [value get];
  BOOL result;
#ifdef INTEL
  fp_pctl pc = fpsetprecision(FP_PC_DBL);
#endif

  result = ((d <= v + epsilon) && (d >= v - epsilon)) ? YES : NO;

#ifdef INTEL
  fpsetprecision(pc);
#endif
  return result;
}

-(BOOL)ne: (DoubleSimple *)value {
  double v = [value get];
  BOOL result;
#ifdef INTEL
  fp_pctl pc = fpsetprecision(FP_PC_DBL);
#endif

  result = ((d > v + epsilon) || (d < v - epsilon)) ? YES : NO;

#ifdef INTEL
  fpsetprecision(pc);
#endif
  return result;
}

-(BOOL)gt: (DoubleSimple *)value {
  double v = [value get];
  BOOL result;
#ifdef INTEL
  fp_pctl pc = fpsetprecision(FP_PC_DBL);
#endif

  result = (d > v + epsilon) ? YES : NO;

#ifdef INTEL
  fpsetprecision(pc);
#endif
  return result;
}

-(BOOL)le: (DoubleSimple *)value {
  double v = [value get];
  BOOL result;
#ifdef INTEL
  fp_pctl pc = fpsetprecision(FP_PC_DBL);
#endif

  result = (d <= v + epsilon) ? YES : NO;

#ifdef INTEL
  fpsetprecision(pc);
#endif
  return result;
}

-(BOOL)ge: (DoubleSimple *)value {
  double v = [value get];
  BOOL result;
#ifdef INTEL
  fp_pctl pc = fpsetprecision(FP_PC_DBL);
#endif

  result = (d >= v - epsilon) ? YES : NO;

#ifdef INTEL
  fpsetprecision(pc);
#endif
  return result;
}

-(BOOL)lt: (DoubleSimple *)value {
  double v = [value get];
  BOOL result;
#ifdef INTEL
  fp_pctl pc = fpsetprecision(FP_PC_DBL);
#endif

  result = (d < v - epsilon) ? YES : NO;

#ifdef INTEL
  fpsetprecision(pc);
#endif
  return result;
}


-(BOOL)ieq: (int)value {
  double v = (double)value;
  BOOL result;
#ifdef INTEL
  fp_pctl pc = fpsetprecision(FP_PC_DBL);
#endif

  result = ((d < v + epsilon) && (d > v - epsilon)) ? YES : NO;

#ifdef INTEL
  fpsetprecision(pc);
#endif
  return result;
}

-(BOOL)ine: (int)value {
  double v = (double)value;
  BOOL result;
#ifdef INTEL
  fp_pctl pc = fpsetprecision(FP_PC_DBL);
#endif

  result = ((d > v + epsilon) || (d < v - epsilon)) ? YES : NO;

#ifdef INTEL
  fpsetprecision(pc);
#endif
  return result;
}

-(BOOL)igt: (int)value {
  double v = (double)value;
  BOOL result;
#ifdef INTEL
  fp_pctl pc = fpsetprecision(FP_PC_DBL);
#endif

  result = (d > v + epsilon) ? YES : NO;

#ifdef INTEL
  fpsetprecision(pc);
#endif
  return result;
}

-(BOOL)ile: (int)value {
  double v = (double)value;
  BOOL result;
#ifdef INTEL
  fp_pctl pc = fpsetprecision(FP_PC_DBL);
#endif

  result = (d < v + epsilon) ? YES : NO;

#ifdef INTEL
  fpsetprecision(pc);
#endif
  return result;
}

-(BOOL)ige: (int)value {
  double v = (double)value;
  BOOL result;
#ifdef INTEL
  fp_pctl pc = fpsetprecision(FP_PC_DBL);
#endif

  result = (d > v - epsilon) ? YES : NO;

#ifdef INTEL
  fpsetprecision(pc);
#endif
  return result;
}

-(BOOL)ilt: (int)value {
  double v = (double)value;
  BOOL result;
#ifdef INTEL
  fp_pctl pc = fpsetprecision(FP_PC_DBL);
#endif

  result = (d < v - epsilon) ? YES : NO;

#ifdef INTEL
  fpsetprecision(pc);
#endif
  return result;
}

-(BOOL)ceq: (double)value {
  BOOL result;
#ifdef INTEL
  fp_pctl pc = fpsetprecision(FP_PC_DBL);
#endif

  result = ((d < value + epsilon) && (d > value - epsilon)) ? YES : NO;

#ifdef INTEL
  fpsetprecision(pc);
#endif
  return result;
}

-(BOOL)cne: (double)value {
  BOOL result;
#ifdef INTEL
  fp_pctl pc = fpsetprecision(FP_PC_DBL);
#endif

  result = ((d > value + epsilon) || (d < value - epsilon)) ? YES : NO;

#ifdef INTEL
  fpsetprecision(pc);
#endif
  return result;
}

-(BOOL)cgt: (double)value {
  BOOL result;
#ifdef INTEL
  fp_pctl pc = fpsetprecision(FP_PC_DBL);
#endif

  result = (d > value + epsilon) ? YES : NO;

#ifdef INTEL
  fpsetprecision(pc);
#endif
  return result;
}

-(BOOL)cle: (double)value {
  BOOL result;
#ifdef INTEL
  fp_pctl pc = fpsetprecision(FP_PC_DBL);
#endif

  result = (d < value + epsilon) ? YES : NO;

#ifdef INTEL
  fpsetprecision(pc);
#endif
  return result;
}

-(BOOL)cge: (double)value {
  BOOL result;
#ifdef INTEL
  fp_pctl pc = fpsetprecision(FP_PC_DBL);
#endif

  result = (d > value - epsilon) ? YES : NO;

#ifdef INTEL
  fpsetprecision(pc);
#endif
  return result;
}

-(BOOL)clt: (double)value {
  BOOL result;
#ifdef INTEL
  fp_pctl pc = fpsetprecision(FP_PC_DBL);
#endif

  result = (d < value - epsilon) ? YES : NO;

#ifdef INTEL
  fpsetprecision(pc);
#endif
  return result;
}

@end
